Summary: The Linux kernel (the core of the Linux operating system)
%define _unpackaged_files_terminate_build 0

# Build Options
%define buildup		1
%define buildsource	0
%define buildheaders	0
%define builddocs	0

# Define
%define sublevel 1
%define kver 3.18
%define kversion %{kver}.%{sublevel}
%define release b1

%define KVERREL %{kversion}-berry

# groups of related archs
%define all_x86 i586 i686

%ifarch %{all_x86}
%define all_arch_configs kernel-%{kversion}-i?86*.config
%define _64bit 0
%endif

%ifarch x86_64
%define all_arch_configs kernel-%{kversion}-x86_64*.config
%define _64bit 1
%endif

# Three sets of minimum package version requirements in the form of Conflicts:
# to versions below the minimum

# First the general kernel 2.6 required versions as per
# Documentation/Changes
%define kernel_dot_org_conflicts ppp < 2.4.3-3, isdn4k-utils < 3.2-32, nfs-utils < 1.0.7-12, e2fsprogs < 1.37-4, util-linux < 2.12, jfsutils < 1.1.7-2, reiserfs-utils < 3.6.19-2, xfsprogs < 2.6.13-4, procps < 3.2.5-6.3, oprofile < 0.9.1-2

# Then a series of requirements that are Red Hat specific, either
# because we add patches for something, or the older versions have
# problems with the newer kernel or lack certain things that make
# integration in the distro harder than needed.
%define package_conflicts initscripts < 7.23, udev < 063-6, iptables < 1.3.2-1, selinux-policy-targeted < 1.25.3-14

# Packages that need to be installed before the kernel is, because the %post
# scripts use them.
%define kernel_prereq	fileutils, module-init-tools

Name: kernel
Group: System Environment/Kernel
License: GPLv2
Version: %{kversion}
Release: %{release}
ExclusiveArch: %{all_x86} x86_64
ExclusiveOS: Linux
Provides: kernel = %{version}
Provides: kernel-drm = 4.3.0, kernel-drm-nouveau
Provides: glibc-kernheaders
Provides: kernel-headers
Requires(pre): %{kernel_prereq}
Conflicts: %{kernel_dot_org_conflicts}
Conflicts: %{package_conflicts}
Conflicts: linux-fitmware
# We can't let RPM do the dependencies automatic because it'll then pick up
# a correct but undesirable perl dependency from the module headers which
# isn't required for the kernel proper to function
AutoReqProv: no
BuildArchitectures: i686

# List the packages used during the kernel build
BuildRequires: module-init-tools, patch >= 2.5.4, bash >= 2.03, sh-utils, tar
BuildRequires: bzip2, findutils, gzip, m4, perl, make >= 3.78
BuildRequires: gcc >= 3.4.2, binutils >= 2.12
BuildConflicts: rhbuildsys(DiskFree) < 500Mb

Source0: ftp://ftp.kernel.org/pub/linux/kernel/v%{kver}/linux-%{kversion}.tar.xz
Source20: berry-%{kver}.config
#Patch1000: linux-2.4.0-test11-vidfail.patch

BuildRoot: %{_tmppath}/kernel-%{KVERREL}-root

%description
The kernel package contains the Linux kernel (vmlinuz), the core ofany
Linux operating system.  The kernel handles the basic functions
of the operating system: memory allocation, process allocation, device
input and output, etc.

%package source
Summary: The source code for the Linux kernel.
Group: Development/System
Requires(pre): fileutils
Requires: make >= 3.78
Requires: gcc >= 3.2
Requires: /usr/bin/strip
# for xconfig and gconfig
#Requires: qt-devel, gtk2-devel readline-devel

%description source
The kernel-source package contains the source code files for the Linux
kernel. The source files can be used to build a custom kernel that is better
tuned to your particular hardware, if you are so inclined (and you know what
you're doing). The customisation guide in the documentation describes in
detail how to do this. This package is not needed nor usable for building
external kernel modules for linking into the default kernel.

%package headers
Summary: Header files for the Linux kernel.
Group: Development/System

%description headers
Kernel-headers includes the C header files for the Linux kernel.  The
header files define structures and constants that are needed for
building most standard programs and are also needed for rebuilding the
kernel.

%package doc
Summary: Various documentation bits found in the kernel source.
Group: Documentation

%description doc
This package contains documentation files from the kernel
source. Various bits of information about the Linux kernel and the
device drivers shipped with it are documented in these files.

You'll want to install this package if you need a reference to the
options that can be passed to Linux kernel modules at load time.


%prep
%setup -q -n %{name}-%{version} -c
cd linux-%{kversion}
#%patch500 -p1
mkdir configs
cp %{SOURCE20} configs/kernel-%{kversion}-%{_target_cpu}.config
#cp %{SOURCE21} configs/kernel-%{kversion}-x86_64.config
cp -fv configs/%{all_arch_configs} .

# now run oldconfig over all the config files
for i in *.config
do
	mv $i .config
	#make ARCH=`echo $i | cut -d"-" -f3 | cut -d"." -f1 | sed -e s/i.86/i386/ -e s/s390x/s390/ -e s/ppc64.series/ppc64/ -e s/ppc8.../ppc/ -e s/ppc32dy4/ppc/ ` nonint_oldconfig > /dev/null
	make oldconfig > /dev/null
	mv .config configs/$i
done

# make sure the kernel has the sublevel we know it has. This looks weird
# but for -pre and -rc versions we need it since we only want to use
# the higher version when the final kernel is released.
perl -p -i -e "s/^SUBLEVEL.*/SUBLEVEL = %{sublevel}/" Makefile
perl -p -i -e "s/^EXTRAVERSION.*/EXTRAVERSION = -prep/" Makefile

# get rid of unwanted files resulting from patch fuzz
find . -name "*.orig" -exec rm -fv {} \;
find . -name "*~" -exec rm -fv {} \;


# build
%build
cd linux-%{kversion}
# prepare directories
rm -rf $RPM_BUILD_ROOT
mkdir -p $RPM_BUILD_ROOT/boot

BuildKernel() {
    # Pick the right config file for the kernel we're building
    if [ -n "$1" ] ; then
	Config=kernel-%{kversion}-%{_target_cpu}-$1.config
    else
	Config=kernel-%{kversion}-%{_target_cpu}.config
    fi

    KernelVer=%{KVERREL}$1
    echo BUILDING A KERNEL FOR $1 %{_target_cpu}...

    # make sure EXTRAVERSION says what we want it to say
    perl -p -i -e "s/^EXTRAVERSION.*/EXTRAVERSION = -berry$1/" Makefile

    # and now to start the build process
    make -s mrproper
    cp configs/$Config .config

    make -s oldconfig > /dev/null
    #make -s include/linux/version.h

    make -s %{?_smp_mflags} bzImage
    make -s %{?_smp_mflags} modules || exit 1

    # Start installing the results
    mkdir -p $RPM_BUILD_ROOT/usr/lib/debug/boot
    install -m 644 System.map $RPM_BUILD_ROOT/usr/lib/debug/boot/System.map-$KernelVer
    objdump -t vmlinux | grep ksymtab | cut -f2 | cut -d" " -f2 | cut -c11- | sort -u  > exported
    echo "_stext" >> exported
    echo "_end" >> exported
    touch $RPM_BUILD_ROOT/boot/System.map-$KernelVer
    for i in `cat exported`
    do
	 grep " $i\$" System.map >> $RPM_BUILD_ROOT/boot/System.map-$KernelVer || :
	 grep "tab_$i\$" System.map >> $RPM_BUILD_ROOT/boot/System.map-$KernelVer || :
	 grep "__crc_$i\$" System.map >> $RPM_BUILD_ROOT/boot/System.map-$KernelVer ||:
    done
    rm -f exported
    install -m 644 .config $RPM_BUILD_ROOT/boot/config-$KernelVer
    rm -f System.map
    rm arch/i386/boot/bzImage
    cp arch/*/boot/bzImage $RPM_BUILD_ROOT/boot/vmlinuz-$KernelVer

    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer
    make -s INSTALL_MOD_PATH=$RPM_BUILD_ROOT modules_install KERNELRELEASE=$KernelVer

    # And save the headers/makefiles etc for building modules against
    #
    # This all looks scary, but the end result is supposed to be:
    # * all arch relevant include/ files
    # * all Makefile/Kconfig files
    # * all script/ files

    rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    # first copy everything
    cp --parents `find -type f -name "Makefile*" -o -name "Kconfig*"` $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    # then drop all but the needed Makefiles/Kconfig files
    rm -rf $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/Documentation
    rm -rf $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/scripts
    rm -rf $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
    ##cp arch/%{_arch}/kernel/asm-offsets.s $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/arch/%{_arch}/kernel || :
    cp arch/x86/kernel/asm-offsets.s $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/arch/x86/kernel || :
    cp .config $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    cp -a scripts $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/scripts/*.o
    rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/scripts/*/*.o

#    if [ -d arch/x86/include ]; then
#      cp -a --parents arch/x86/include $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/
#    fi
    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
    cd include
    # NOTE: deleted config, asm, asm-x86 dirs
    cp -a acpi keys linux math-emu media net pcmcia rdma rxrpc scsi sound trace uapi video drm asm-generic xen $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
#    cp -a `readlink asm` $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
    ln -s /lib/modules/$KernelVer/build/arch/x86/include/asm $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
    # Make sure the Makefile and version.h have a matching timestamp so that
    # external modules can be built
    #touch -r $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/Makefile $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include/linux/version.h
    touch -r $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/.config $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include/linux/autoconf.h
    cd ..
    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include/config
    cp -a include/config/{auto.conf*,kernel.release} $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include/config/
    # since 2.6.33
    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include/generated/
    cp -a include/generated/* $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include/generated/
    rm $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include/linux/autoconf.h
    ln -s /lib/modules/$KernelVer/build/include/generated/autoconf.h $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include/linux/autoconf.h
    ln -s /lib/modules/$KernelVer/build/include/generated/utsrelease.h $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include/linux/utsrelease.h
    #cp -a arch/x86/include/asm/* $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include/asm-x86/
    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/arch/x86/
    cp -a arch/x86/include $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/arch/x86/

    # save the vmlinux file for kernel debugging into the kernel-debuginfo rpm
#%if %{with_debuginfo}
    mkdir -p $RPM_BUILD_ROOT/usr/lib/debug/lib/modules/$KernelVer
    #cp vmlinux $RPM_BUILD_ROOT/usr/lib/debug/lib/modules/$KernelVer
    eu-strip --remove-comment -o $RPM_BUILD_ROOT/usr/lib/debug/lib/modules/$KernelVer/vmlinux vmlinux
#%endif

    # mark modules executable so that strip-to-file can strip them
    find $RPM_BUILD_ROOT/lib/modules/$KernelVer -name "*.ko*" -type f | xargs chmod u+x

    # remove files that will be auto generated by depmod at rpm -i time
    rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/modules.*
}

%if %{buildup}
BuildKernel
%endif


# install
%install
cd linux-%{kversion}

%if %{buildsource}
mkdir -p $RPM_BUILD_ROOT/usr/src/linux-%{KVERREL}

# clean up the source tree so that it is ready for users to build their own kernel
make -s mrproper
# copy the source over
tar cf - . | tar xf - -C $RPM_BUILD_ROOT/usr/src/linux-%{KVERREL}

# set the EXTRAVERSION to <version>custom, so that people who follow a kernel building howto
# don't accidentally overwrite their currently working moduleset and hose their system
perl -p -i -e "s/^EXTRAVERSION.*/EXTRAVERSION = -%{release}/" $RPM_BUILD_ROOT/usr/src/linux-%{KVERREL}/Makefile
#install -m 644 %{SOURCE10} $RPM_BUILD_ROOT/usr/src/linux-%{KVERREL}

mkdir -p $RPM_BUILD_ROOT/usr/src
ln -sf /usr/src/linux-%{KVERREL} $RPM_BUILD_ROOT/usr/src/linux-%{KVERREL}
%endif

# /usr/include
mkdir -p $RPM_BUILD_ROOT/usr/include
#ln -sf /lib/modules/%{KVERREL}/build/include/linux $RPM_BUILD_ROOT/usr/include/linux
#ln -sf /lib/modules/%{KVERREL}/build/include/trace $RPM_BUILD_ROOT/usr/include/trace
ln -sf /lib/modules/%{KVERREL}/build/include/xen $RPM_BUILD_ROOT/usr/include/xen
#ln -sf /lib/modules/%{KVERREL}/build/include/asm-generic $RPM_BUILD_ROOT/usr/include/asm-generic
#ln -sf /lib/modules/%{KVERREL}/build/arch/x86/include/asm $RPM_BUILD_ROOT/usr/include/asm
ln -sf /lib/modules/%{KVERREL}/build/arch/x86/include/uapi/asm $RPM_BUILD_ROOT/usr/include/asm

ln -sf /lib/modules/%{KVERREL}/build/include/uapi/asm-generic $RPM_BUILD_ROOT/usr/include/asm-generic
ln -sf /lib/modules/%{KVERREL}/build/include/uapi/linux $RPM_BUILD_ROOT/usr/include/linux
ln -sf /lib/modules/%{KVERREL}/build/include/uapi $RPM_BUILD_ROOT/usr/include/uapi

ln -sf /lib/modules/%{KVERREL}/build/include/linux/compiler.h $RPM_BUILD_ROOT/lib/modules/%{KVERREL}/build/include/uapi/linux
#ln -sf /lib/modules/%{KVERREL}/build/include/linux/*.h $RPM_BUILD_ROOT/lib/modules/%{KVERREL}/build/include/uapi/linux
ln -sf /lib/modules/%{KVERREL}/build/include/generated/uapi/linux/version.h $RPM_BUILD_ROOT/lib/modules/%{KVERREL}/build/include/uapi/linux
ln -sf /lib/modules/%{KVERREL}/build/include/generated/autoconf.h $RPM_BUILD_ROOT/lib/modules/%{KVERREL}/build/include/uapi/linux

ln -sf /lib/modules/%{KVERREL}/build/arch/x86/include/generated/uapi/asm/*.h $RPM_BUILD_ROOT/lib/modules/%{KVERREL}/build/arch/x86/include/uapi/asm/
#cp -aus $RPM_BUILD_ROOT/usr/include/uapi/linux/*.h $RPM_BUILD_ROOT/usr/include/linux
#rm `find . -type l`
##cp -a $RPM_BUILD_ROOT/usr/include/* $RPM_BUILD_ROOT/usr/include/

# glibc provides scsi headers for itself, for now
#rm -rf $RPM_BUILD_ROOT/usr/include/scsi
#rm -f $RPM_BUILD_ROOT/usr/include/asm*/atomic.h
#rm -f $RPM_BUILD_ROOT/usr/include/asm*/io.h
#rm -f $RPM_BUILD_ROOT/usr/include/asm*/irq.h


# clean
%clean
rm -rf $RPM_BUILD_ROOT

# scripts
%post
cd /boot
ln -sf vmlinuz-%{KVERREL} vmlinuz
ln -sf System.map-%{KVERREL} System.map
depmod -a -F /boot/System.map-%{KVERREL} %{KVERREL}

%preun
rm -f /lib/modules/%{KVERREL}/modules.*


# file lists
%if %{buildup}
%files
%defattr(-,root,root)
/boot/vmlinuz-%{KVERREL}
/boot/System.map-%{KVERREL}
#/boot/config-%{KVERREL}
%dir /lib/modules/%{KVERREL}
/lib/modules/%{KVERREL}/kernel
%verify(not mtime) /lib/modules/%{KVERREL}/build
/usr/include/asm
/usr/include/asm-generic
/usr/include/linux
/usr/include/uapi
/usr/include/xen
/lib/firmware/
%endif

# only some architecture builds need kernel-source and kernel-doc
%if %{buildsource}

%files source
%defattr(-,root,root)
%dir /usr/src/linux-%{KVERREL}
/usr/src/linux-%{KVERREL}/COPYING
/usr/src/linux-%{KVERREL}/CREDITS
/usr/src/linux-%{KVERREL}/MAINTAINERS
/usr/src/linux-%{KVERREL}/Makefile
/usr/src/linux-%{KVERREL}/README
/usr/src/linux-%{KVERREL}/REPORTING-BUGS
/usr/src/linux-%{KVERREL}/arch/x86
/usr/src/linux-%{KVERREL}/crypto
/usr/src/linux-%{KVERREL}/drivers
/usr/src/linux-%{KVERREL}/fs
/usr/src/linux-%{KVERREL}/init
/usr/src/linux-%{KVERREL}/ipc
/usr/src/linux-%{KVERREL}/kernel
/usr/src/linux-%{KVERREL}/lib
/usr/src/linux-%{KVERREL}/mm
/usr/src/linux-%{KVERREL}/net
/usr/src/linux-%{KVERREL}/scripts
/usr/src/linux-%{KVERREL}/security
/usr/src/linux-%{KVERREL}/sound
/usr/src/linux-%{KVERREL}/usr
/usr/src/linux-%{KVERREL}/include/asm-x86
#/usr/src/linux-%{KVERREL}/include/asm
/usr/src/linux-%{KVERREL}/include/asm-generic
/usr/src/linux-%{KVERREL}/include/linux
/usr/src/linux-%{KVERREL}/include/acpi
/usr/src/linux-%{KVERREL}/include/math-emu
/usr/src/linux-%{KVERREL}/include/media
#/usr/src/linux-%{KVERREL}/include/mtd
/usr/src/linux-%{KVERREL}/include/net
/usr/src/linux-%{KVERREL}/include/pcmcia
/usr/src/linux-%{KVERREL}/include/rxrpc
/usr/src/linux-%{KVERREL}/include/scsi
/usr/src/linux-%{KVERREL}/include/sound
/usr/src/linux-%{KVERREL}/include/uapi
/usr/src/linux-%{KVERREL}/include/video
/usr/src/linux-%{KVERREL}/include/xen
/usr/src/linux-2.6

%if %{buildheaders}
%files headers
%defattr(-,root,root)
/usr/include/asm
/usr/include/asm-generic
/usr/include/linux
/usr/include/xen
#/usr/src/linux-%{KVERREL}/include/asm-i386
#/usr/src/linux-%{KVERREL}/include/asm-generic
#/usr/src/linux-%{KVERREL}/include/linux
%verify(not mtime) /lib/modules/%{KVERREL}/build
%endif

%if %{builddocs}
%files doc
%defattr(-,root,root)
%doc /usr/src/linux-%{KVERREL}/Documentation/*
%endif

%endif


# change log
%changelog
* Sat Dec 20 2014 Yuichiro Nakada <berry@berry-lab.net>
- Update to 3.18.1
* Mon Sep 8 2014 Yuichiro Nakada <berry@berry-lab.net>
- Update to 3.16.2
* Thu May 29 2014 Yuichiro Nakada <berry@berry-lab.net>
- Update to 3.14.4
* Mon Jan 6 2014 Yuichiro Nakada <berry@berry-lab.net>
- Update to 3.12.6
* Wed May 15 2013 Yuichiro Nakada <berry@berry-lab.net>
- Update to 3.9.2
* Fri Dec 28 2012 Yuichiro Nakada <berry@berry-lab.net>
- Update to 3.7.1
* Thu Aug 23 2012 Yuichiro Nakada <berry@rberry.co.cc>
- Update to 3.5.2
* Wed Jun 6 2012 Yuichiro Nakada <berry@rberry.co.cc>
- Update to 3.4.1
* Tue Apr 10 2012 Yuichiro Nakada <berry@rberry.co.cc>
- Update to 3.3.1
* Mon Feb 13 2012 Yuichiro Nakada <berry@rberry.co.cc>
- Update to 3.2.5
* Thu Dec 1 2011 Yuichiro Nakada <berry@rberry.co.cc>
- Update to 3.1.4
* Thu Jul 28 2011 Yuichiro Nakada <berry@rberry.co.cc>
- Update to 3.0
* Mon May 23 2011 Yuichiro Nakada <berry@rberry.co.cc>
- Update to 2.6.39
* Mon Jan 12 2004 Yuichiro Nakada <berry@po.yui.mine.nu>
- Update to 2.6.1
* Sat Dec 20 2003 Yuichiro Nakada <berry@po.yui.mine.nu>
- Create for Berry Linux
- from Red Hat Fedora Core 1
